﻿# Author: fweixelb@frequenits.com
# Date: 02.04.2015
# Description: Alarmhandler for vCenter/Zabbix - Script forwards all alarms to Zabbix NMS which become active in vCenter as far as the script is defined in the runcommand of the alert
#
#
#
#
# Adds the base cmdlets
Add-PSSnapin VMware.VimAutomation.Core


#output for log - log can be defined in redirect.bat by adding "> $PathToLog"
#eg: echo.|powershell -NonInteractive -File C:\tools\alarmHandler.ps1 >> C:\tools\alarmHandler.log
echo ""
echo ""
echo "********************************************************************"
echo $(Get-Date)
echo "**********"
echo ""

#functions:

function Get-OSArchitecture {                      
      
$ComputerName = $env:computername 
 foreach ($Computer in $ComputerName) {            
  if(Test-Connection -ComputerName $Computer -Count 1 -ea 0) {            
   Write-Verbose "$Computer is online"            
   $OS  = (Get-WmiObject -computername $computer -class Win32_OperatingSystem ).Caption            
   if ((Get-WmiObject -Class Win32_OperatingSystem -ComputerName $Computer -ea 0).OSArchitecture -eq '64-bit') {            
    $architecture = "win64"            
   } else  {            
    $architecture = "win32"            
   }            
   $architecture           
  }            
 }            
} 

Function Replace-Umlaut {

    param (
       [string]$Text 
    )
 
    # create Table with replace map
    $characterMap = @{}
    $characterMap.([Int][Char]'ä') = "ae"
    $characterMap.([Int][Char]'ö') = "oe"
    $characterMap.([Int][Char]'ü') = "ue"
    $characterMap.([Int][Char]'ß') = "ss"
    $characterMap.([Int][Char]'Ä') = "Ae"
    $characterMap.([Int][Char]'Ü') = "Ue"
    $characterMap.([Int][Char]'Ö') = "Oe"
    $characterMap.([Int][Char]'á') = "_"
    $characterMap.([Int][Char]"`n") = " "
    
    # Replace chars
    ForEach ($key in $characterMap.Keys) {
        $Text = $Text -creplace ([Char]$key),$characterMap[$key] 
    }
 
     # return result
     $Text
}
Function Replace-Umlaut-CR {

    param (
       [string]$Text 
    )
 
    # create Table with replace map
    $characterMap = @{}
    $characterMap.([Int][Char]'ä') = "ae"
    $characterMap.([Int][Char]'ö') = "oe"
    $characterMap.([Int][Char]'ü') = "ue"
    $characterMap.([Int][Char]'ß') = "ss"
    $characterMap.([Int][Char]'Ä') = "Ae"
    $characterMap.([Int][Char]'Ü') = "Ue"
    $characterMap.([Int][Char]'Ö') = "Oe"
    $characterMap.([Int][Char]'á') = "_"
    #$characterMap.([Int][Char]"`n") = " "
    
    # Replace chars
    ForEach ($key in $characterMap.Keys) {
        $Text = $Text -creplace ([Char]$key),$characterMap[$key] 
    }
 
     # return result
     $Text
}


Function Format-Status {

    param (
       [string]$Status 
    )
    $Status = Replace-Umlaut $Status.ToLower();
            
    echo $Status > C:\tools\status.txt;

    switch($Status)
    {
        "gruen" {$Status=0; break}
        "grau" { $Status=1; break}
        "gelb" { $Status=2; break}
        "rot" { $Status=3; break}
        "green" { $Status=0; break}
        "gray" { $Status=1; break}
        "yellow" { $Status=2; break}
        "red" { $Status=3; break}
    }
        
    # return result
    $Status
}

###################################################################################################################################
#Var declaration
#the follwing variables can be adapted to your needs

#ATTENTION: after initial createion of a ne item by our discovery rule zabbis will need about 2 minutes to be ready to receive data
# so the reties multiplied by the retry-timeout should be over 120 to ensure data are sent even for the first alarm
#retries to send data if not processed by zabbix server
$sendRetries=25
#timout after each retry in seconds
$retryTimeout=5

#Path to the zabbix-agent configuration file
$zabbixAgentConf="C:\zabbixAgent\conf\zabbix_agentd.win.conf";
#Path to Zabbix sender executable
$arch=Get-OSArchitecture
$zabbixSenderExe="C:\zabbixAgent\bin\" + $arch + "\zabbix_sender.exe";
#Name of the item for the discovery - if it is changed here it also has to be changed in zabbix
$itemKey="vmware.alarm.discovery"

#path where temporary containers are stored
#ATTENTION:Do not store on C: - if files are not deleted for any reason the c-partition will run full and kill the system!
$containerFilePath="D:\ZabbixVCenter"
#Prefix for temp container files
$tempValueContainerFile=$containerFilePath+"\VcAlarmVal_"


#do not change vars down here if you do not know what you are doing!
#echo $zabbixSenderExe >> C:\tools\tmp.txt
#Write environmental vars to internal vars
$ComputerName=$env:COMPUTERNAME
$VmwareAlarmTargetId=$env:VMWARE_ALARM_TARGET_ID
$VmwareAlarmId=$env:VMWARE_ALARM_ID
$VmwareAlarmName=$env:VMWARE_ALARM_NAME
$VmwareAlarmOldStatus=$env:VMWARE_ALARM_OLDSTATUS
$VmwareAlarmNewStatus=$env:VMWARE_ALARM_NEWSTATUS
$VmwareAlarmEventDescription=$env:VMWARE_ALARM_EVENTDESCRIPTION
$VmwareAlarmTargetName=$env:VMWARE_ALARM_TARGET_NAME
$timestamp=[int][double]::Parse((Get-Date -UFormat %s))
$sent=0; $count=0;

###################################################################################################################################

#create directory for temp container files if dir does not exist
if(!(Test-Path $containerFilePath))
{
    New-Item -ItemType directory -Path $containerFilePath
}


#Send discovery to zabbix
#Data
#Key: VMWARE_ALARM_TARGET_ID-VMWARE_ALARM_ID
#VMWARE_ALARM_NAME              test_host 
#VMWARE_ALARM_NEWSTATUS         Gelb
#VMWARE_ALARM_OLDSTATUS         Grau    
#VMWARE_ALARM_EVENTDESCRIPTION  Alarm 'test_host' auf cube-esx-02.cube.demo w...
#VMWARE_ALARM_TARGET_NAME       cube-esx-02.cube.demo 

$jsonDiscoveryData=$ComputerName + "`t" + $itemKey + "`t" + '{ "data":[ { "{#KEY}" : "'+ $VmwareAlarmTargetId +'-'+ $VmwareAlarmId +'", "{#ALARM_NAME}" : "'+ $VmwareAlarmName +'", "{#ALARM_OLD_STATUS}" : "'+ $VmwareAlarmOldStatus +'", "{#ALARM_NEW_STATUS}" : "'+ $VmwareAlarmNewStatus +'", "{#ALARM_EVENT_DESC}" : "'+ $VmwareAlarmEventDescription +'", "{#ALARM_TARGET_NAME}" : "'+ $VmwareAlarmTargetName +'"} ] }';
$c=Replace-Umlaut $jsonDiscoveryData
#write data temporary to a file which is passed to zabbix sender 
New-Item $tempValueContainerFile$timestamp -Type file -Force -Value $c
$exec=($zabbixSenderExe + " -i " + $tempValueContainerFile+$timestamp + " -v -c "+$zabbixAgentConf)
#echo $jsonDiscoveryData
#echo $exec
$ret = Invoke-Expression $exec
echo $ret;
$serverResponse=$ret[0];
#if sending of the discovery fails -> do not try to send the data for the discovered items because there is nothing to send!
if($serverResponse.contains("failed: 0;"))
{
    $sent=0;
} else {
    $sent=1;
}
$ret=""; $serverResponse="";
#we are throu - delete the file - for debugging purpose the following line could be uncommented
Remove-Item $tempValueContainerFile$timestamp -Force

#echo $c >> C:\tools\tmp.txt
###################################################################################################################################


#if items got discovered - send the data to zabbix
#Sleep for a while to make sure that the Items are already discoverd when we want to send data
#Start-Sleep -Seconds 120


#prepare: last status
$key="alarm["+ $VmwareAlarmTargetId +"-"+ $VmwareAlarmId +"_oldstatus]";
$value=Format-Status $VmwareAlarmOldStatus;
$oldState="$ComputerName`t$key`t$value";
#prepare: current status
$key="alarm["+ $VmwareAlarmTargetId +"-"+ $VmwareAlarmId +"_newstatus]";
$value=Format-Status $VmwareAlarmNewStatus;
$newState="$ComputerName`t$key`t$value";
#prepare: description
$key="alarm["+ $VmwareAlarmTargetId +"-"+ $VmwareAlarmId +"_description]";
$value=$VmwareAlarmEventDescription;
$description="$ComputerName`t$key`t`"$value`"";
#collet all the data in one var which is written to a file
#$sendItems=Replace-Umlaut-CR ($oldState+"`n"+$newState+"`n"+$description)
$sendItems=Replace-Umlaut-CR ($newState+"`n"+$description)
#write data temporary to a file which is passed to zabbix sender 
New-Item $tempValueContainerFile$timestamp -Type file -Force -Value $sendItems
$exec=($zabbixSenderExe + " -i " + $tempValueContainerFile+$timestamp + " -v -c "+$zabbixAgentConf)

#send data to zabbix till we get 0 failed items or we tried over 120 times
do{
    $ret=Invoke-Expression $exec
    echo $ret
    #we are only interested in the first line of the zabbix-sender response
    $serverResponse=$ret[0]
    #check if everything ran through or if the attemp failed -> if failed - retry and waoit for $retryTimeout seconds befor next zabbix-sender call
    #ATTENTION: if an item is disabled this item will fail and the alert handler will contunue sending data
    #Items must be also disalbed in the alert handler
    if($serverResponse.contains("failed: 0;"))
    {
        $sent=1;
    } else {
        $sent=0;
        Start-Sleep -Seconds $retryTimeout
    }
    echo $sent
    $count++
    Write-Host -ForegroundColor red $count
    
    #exit the loop if sending is not possible and we are stuck
    if($count>$sendRetries)
    {
        $sent=1;
    }
}
while($sent -eq 0)
#we are throu - delete the file - for debugging purpose the following line could be uncommented
Remove-Item $tempValueContainerFile$timestamp -Force