﻿# Author: fweixelb@frequenits.com
# Date: 03.04.2015
# Description: Update "Run Command" of vCenter Alarms with a specific script command
#
#
#
param (
  [Parameter(Mandatory=$False)]
   [string] $zabbixHostName = $env:COMPUTERNAME,
	
   [Parameter(Mandatory=$false)]
   [string] $username = "administrator@vsphere.local",

   [Parameter(Mandatory=$false)]
   [string] $password = "password",

   [Parameter(Mandatory=$false)]
   [string] $vCenterServer = "127.0.0.1",
	
   [Parameter(Mandatory=$False)]
   [string] $scriptPath = "C:\zabbixAgent\vCenterAlerting\redirect.bat",
   
   #if old Entrys shall be deleted
   [Parameter(Mandatory=$False)]
   [int16] $delteOld = 1
)
echo "Create alerts for Zabbix in vCenter alarms"
# Adds the base cmdlets
Add-PSSnapin VMware.VimAutomation.Core

function setZabbixAlertHandler() {

	$connection=Connect-VIServer $vCenterServer -User $username -Password $password

    if($connection)
    {
        echo "connected"
    } else {
        echo "not connected"
    }

	$alarms = Get-AlarmDefinition
    foreach( $alarm in $alarms)
    {
        echo "handling alarm: "$alarm.Name
        
        
        Check-Alarms($alarm)

        #select only one alarm - debug/testing
        if($alarm.name -contains "Lizenzbestandsüberwachung")
        {
            

        }
    }
    Disconnect-VIServer -Confirm:$false

	#
    #echo $alarmDefinition
    #n
    #echo $alarmAction
    
    #
    #Remove-Item $exportFolder\alarms.txt -Force
}

Function Replace-Umlaut {

    param (
       [string]$Text 
    )
 
    # create Table with replace map
    $characterMap = @{}
    $characterMap.([Int][Char]'ä') = "ae"
    $characterMap.([Int][Char]'ö') = "oe"
    $characterMap.([Int][Char]'ü') = "ue"
    $characterMap.([Int][Char]'ß') = "ss"
    $characterMap.([Int][Char]'Ä') = "Ae"
    $characterMap.([Int][Char]'Ü') = "Ue"
    $characterMap.([Int][Char]'Ö') = "Oe"
    $characterMap.([Int][Char]'á') = "_"
    $characterMap.([Int][Char]"`n") = " "
    
    # Replace chars
    ForEach ($key in $characterMap.Keys) {
        $Text = $Text -creplace ([Char]$key),$characterMap[$key] 
    }
 
     # return result
     $Text
}



Function Create-Key {
    param (
       [string]$Text 
    )
    
    $hasher = new-object System.Security.Cryptography.SHA256Managed
    $toHash = [System.Text.Encoding]::UTF8.GetBytes($Text)
    $hashByteArray = $hasher.ComputeHash($toHash)
    foreach($byte in $hashByteArray)
    {
         $ret += $byte.ToString()
    }
     # return result
     $Text=$ret
     $text
}

Function Check-Alarms($alarm) {

    #Set the Script to run
    $actionExists=Get-AlarmDefinition -Id $alarm.Id | Get-AlarmAction -ActionType ExecuteScript
    if($actionExists.ActionType -eq "ExecuteScript")
    {
        #echo $actionExists
        if($delteOld -eq 1)
        {
            Remove-AlarmAction -AlarmAction $actionExists -Confirm:$False 
            Create-Alarms($alarm)
        } else {
           echo "Alarm: "$alarm.Name" already exists! - To delete the old action and set a new one pass delteOld=1"
        }

    } else {

        Create-Alarms($alarm)
    }
 
 
}

function Create-Alarms($alarm) {
    $alarmAction=Get-AlarmDefinition -id $alarm.Id | New-AlarmAction -Script -ScriptPath "`"$scriptPath`""
    #Set the Triggers when the alarm is run
    #$alarmActionTrigger=Get-AlarmActionTrigger -AlarmAction $alarmAction
    New-AlarmActionTrigger -StartStatus 'Green' -EndStatus 'Yellow' -Repeat -AlarmAction $alarmAction
    New-AlarmActionTrigger -StartStatus 'Red' -EndStatus 'Yellow' -Repeat -AlarmAction $alarmAction
    New-AlarmActionTrigger -StartStatus 'Yellow' -EndStatus 'Green' -Repeat -AlarmAction $alarmAction
     
    #delte default trigger & set to repeat
    #Get-AlarmDefinition -id $alarm.Id | Get-AlarmAction -ActionType ExecuteScript | Get-AlarmActionTrigger | select -First 1 | Remove-AlarmActionTrigger
    $yellowRedTrigger=Get-AlarmActionTrigger -AlarmAction $alarmAction | select -First 1
    Remove-AlarmActionTrigger -AlarmActionTrigger $yellowRedTrigger -Confirm:$False
    New-AlarmActionTrigger -StartStatus 'Yellow' -EndStatus 'Red' -Repeat -AlarmAction $alarmAction
}


setZabbixAlertHandler